@@ -69,6 +69,7 @@ struct _GstSchroEnc
 
   /* state */
   gboolean got_offset;
+  gboolean update_granulepos;
   uint64_t granulepos_offset;
   uint64_t granulepos_low;
   uint64_t granulepos_hi;
@@ -223,6 +224,7 @@ gst_schro_enc_sink_setcaps (GstPad *pad, GstCaps *caps)
 {
   GstStructure *structure;
   GstSchroEnc *schro_enc = GST_SCHRO_ENC (gst_pad_get_parent (pad));
+  GstCaps *srccaps;
 
   structure = gst_caps_get_structure (caps, 0);
 
@@ -276,6 +278,20 @@ gst_schro_enc_sink_setcaps (GstPad *pad, GstCaps *caps)
   schro_enc->duration = gst_util_uint64_scale_int (GST_SECOND,
           schro_enc->fps_d, schro_enc->fps_n);
 
+  srccaps = gst_caps_new_simple ("video/x-dirac",
+              "width", G_TYPE_INT, schro_enc->width,
+              "height", G_TYPE_INT, schro_enc->height,
+              "framerate", GST_TYPE_FRACTION, schro_enc->fps_n,
+              schro_enc->fps_d,
+              NULL);
+
+  if (schro_enc->par_d != 1 || schro_enc->par_n != 1)
+    gst_caps_set_simple (srccaps, "pixel-aspect-ratio", GST_TYPE_FRACTION, schro_enc->par_n,
+	      schro_enc->par_d, NULL);
+
+  gst_pad_set_caps (schro_enc->srcpad, srccaps);
+  gst_caps_unref (srccaps);
+
   gst_object_unref (GST_OBJECT(schro_enc));
 
   return TRUE;
@@ -435,7 +451,7 @@ gst_schro_buffer_wrap (GstSchroEnc *schro_enc, GstBuffer *buf,
   return frame;
 }
 
-#define OGG_DIRAC_GRANULE_SHIFT 30
+#define OGG_DIRAC_GRANULE_SHIFT 32
 #define OGG_DIRAC_GRANULE_LOW_MASK ((1ULL<<OGG_DIRAC_GRANULE_SHIFT)-1)
 
 static gint64
@@ -638,6 +654,7 @@ gst_schro_enc_chain (GstPad *pad, GstBuffer *buf)
 
     GST_DEBUG("using granulepos offset %lld", schro_enc->granulepos_offset);
     schro_enc->granulepos_hi = 0;
+    schro_enc->granulepos_low = 0;
     schro_enc->got_offset = TRUE;
 
     schro_enc->timestamp_offset = GST_BUFFER_TIMESTAMP(buf);
@@ -685,31 +702,30 @@ gst_schro_enc_process (GstSchroEnc *schro_enc)
         parse_code = encoded_buffer->data[4];
 
         if (SCHRO_PARSE_CODE_IS_SEQ_HEADER(parse_code)) {
-          schro_enc->granulepos_hi = schro_enc->granulepos_offset +
-            presentation_frame + 1;
+          schro_enc->update_granulepos = TRUE;
+        }
+        if (SCHRO_PARSE_CODE_IS_PICTURE(parse_code)) {
+          if (schro_enc->update_granulepos) {
+            schro_enc->granulepos_hi = schro_enc->granulepos_offset +
+              presentation_frame + 1;
+            schro_enc->update_granulepos = FALSE;
+          }
+          schro_enc->granulepos_low = schro_enc->granulepos_offset +
+            presentation_frame + 1 - schro_enc->granulepos_hi;
         }
-
-        schro_enc->granulepos_low = schro_enc->granulepos_offset +
-          presentation_frame + 1 - schro_enc->granulepos_hi;
 
         outbuf = gst_buffer_new_and_alloc (encoded_buffer->length);
         memcpy (GST_BUFFER_DATA (outbuf), encoded_buffer->data,
             encoded_buffer->length);
-        gst_buffer_set_caps (outbuf,
-            gst_caps_new_simple ("video/x-dirac",
-              "width", G_TYPE_INT, schro_enc->width,
-              "height", G_TYPE_INT, schro_enc->height,
-              "framerate", GST_TYPE_FRACTION, schro_enc->fps_n,
-              schro_enc->fps_d,
-              NULL));
+        gst_buffer_set_caps (outbuf, GST_PAD_CAPS (schro_enc->srcpad));
 
+        GST_BUFFER_OFFSET_END (outbuf) =
+          (schro_enc->granulepos_hi<<OGG_DIRAC_GRANULE_SHIFT) +
+          schro_enc->granulepos_low;
+        GST_BUFFER_OFFSET (outbuf) = gst_util_uint64_scale (
+            (schro_enc->granulepos_hi + schro_enc->granulepos_low),
+            schro_enc->fps_d * GST_SECOND, schro_enc->fps_n);
         if (SCHRO_PARSE_CODE_IS_PICTURE(parse_code)) {
-          GST_BUFFER_OFFSET_END (outbuf) =
-            (schro_enc->granulepos_hi<<OGG_DIRAC_GRANULE_SHIFT) +
-            schro_enc->granulepos_low;
-          GST_BUFFER_OFFSET (outbuf) = gst_util_uint64_scale (
-              (schro_enc->granulepos_hi + schro_enc->granulepos_low),
-              schro_enc->fps_d * GST_SECOND, schro_enc->fps_n);
           GST_BUFFER_DURATION (outbuf) = schro_enc->duration;
           GST_BUFFER_TIMESTAMP (outbuf) = 
             schro_enc->timestamp_offset + gst_util_uint64_scale (
@@ -718,16 +734,16 @@ gst_schro_enc_process (GstSchroEnc *schro_enc)
           schro_enc->picture_number++;
           if (!SCHRO_PARSE_CODE_IS_INTRA(parse_code)) {
             GST_BUFFER_FLAG_SET (outbuf, GST_BUFFER_FLAG_DELTA_UNIT);
+          } else {
+            GST_BUFFER_FLAG_UNSET (outbuf, GST_BUFFER_FLAG_DELTA_UNIT);
           }
         } else {
-          GST_BUFFER_OFFSET_END (outbuf) = 0;
-          GST_BUFFER_OFFSET (outbuf) = 0;
           GST_BUFFER_DURATION (outbuf) = -1;
-          //GST_BUFFER_TIMESTAMP (outbuf) = -1;
           GST_BUFFER_TIMESTAMP (outbuf) = 
             schro_enc->timestamp_offset + gst_util_uint64_scale (
               schro_enc->picture_number,
               schro_enc->fps_d * GST_SECOND, schro_enc->fps_n);
+          GST_BUFFER_FLAG_UNSET (outbuf, GST_BUFFER_FLAG_DELTA_UNIT);
         }
 
         GST_INFO("size %d offset %lld granulepos %llu:%llu timestamp %lld duration %lld",
