@@ -28,6 +28,7 @@ struct _SchroPictureSubbandContext {
   int component;
   int index;
   int position;
+  int broken;
 
   SchroFrameData *frame_data;
   SchroFrameData *parent_frame_data;
@@ -149,7 +150,7 @@ schro_decoder_free (SchroDecoder *decoder)
   if (decoder->cpu_domain) schro_memory_domain_free (decoder->cpu_domain);
   if (decoder->cuda_domain) schro_memory_domain_free (decoder->cuda_domain);
 
-  if (decoder->access_unit_buffer) schro_buffer_unref (decoder->access_unit_buffer);
+  if (decoder->sequence_header_buffer) schro_buffer_unref (decoder->sequence_header_buffer);
 
   schro_free (decoder);
 }
@@ -170,7 +171,9 @@ schro_picture_new (SchroDecoder *decoder)
   SchroPicture *picture;
   SchroFrameFormat frame_format;
   SchroVideoFormat *video_format = &decoder->video_format;
-  int frame_width, frame_height;
+  int picture_width, picture_height;
+  int iwt_width, iwt_height;
+  int picture_chroma_width, picture_chroma_height;
 
   picture = schro_malloc0 (sizeof(SchroPicture));
   picture->refcount = 1;
@@ -183,14 +186,18 @@ schro_picture_new (SchroDecoder *decoder)
 
   frame_format = schro_params_get_frame_format (16,
       video_format->chroma_format);
-  frame_width = ROUND_UP_POW2(video_format->width,
-      SCHRO_LIMIT_TRANSFORM_DEPTH + video_format->chroma_h_shift);
-  frame_height = ROUND_UP_POW2(video_format->height,
-      SCHRO_LIMIT_TRANSFORM_DEPTH + video_format->chroma_v_shift);
+  schro_video_format_get_picture_chroma_size (video_format,
+      &picture_chroma_width, &picture_chroma_height);
+
+  picture_width = video_format->width;
+  picture_height = schro_video_format_get_picture_height (video_format);
+
+  schro_video_format_get_iwt_alloc_size (video_format, &iwt_width,
+      &iwt_height);
 
   if (decoder->use_cuda) {
     picture->transform_frame = schro_frame_new_and_alloc (decoder->cpu_domain,
-        frame_format, frame_width, frame_height);
+        frame_format, iwt_width, iwt_height);
 #if 0
     /* These get allocated later, while in the CUDA thread */
     picture->mc_tmp_frame = schro_frame_new_and_alloc (decoder->cuda_domain,
@@ -202,9 +209,9 @@ schro_picture_new (SchroDecoder *decoder)
 #endif
   } else {
     picture->mc_tmp_frame = schro_frame_new_and_alloc (decoder->cpu_domain,
-        frame_format, frame_width, frame_height);
+        frame_format, picture_width, picture_height);
     picture->frame = schro_frame_new_and_alloc (decoder->cpu_domain,
-        frame_format, frame_width, frame_height);
+        frame_format, iwt_width, iwt_height);
     picture->transform_frame = schro_frame_ref (picture->frame);
 
     picture->planar_output_frame = schro_frame_new_and_alloc (decoder->cpu_domain,
@@ -289,12 +296,15 @@ schro_decoder_reset (SchroDecoder *decoder)
   schro_queue_clear (decoder->reference_queue);
   schro_queue_clear (decoder->output_queue);
 
-  decoder->have_access_unit = FALSE;
+  decoder->have_sequence_header = FALSE;
   decoder->next_frame_number = 0;
   decoder->have_frame_number = FALSE;
 
   decoder->end_of_stream = FALSE;
+  decoder->flushing = FALSE;
   schro_async_unlock (decoder->async);
+
+  decoder->error = FALSE;
 }
 
 /**
@@ -413,16 +423,30 @@ schro_decoder_set_skip_ratio (SchroDecoder *decoder, double ratio)
   decoder->skip_ratio = ratio;
 }
 
+void
+schro_decoder_set_picture_order (SchroDecoder *decoder, int order)
+{
+  if (order == SCHRO_DECODER_PICTURE_ORDER_CODED) {
+    decoder->coded_order = TRUE;
+  } else {
+    decoder->coded_order = FALSE;
+  }
+}
 
 static int
 schro_decoder_pull_is_ready_locked (SchroDecoder *decoder)
 {
   SchroPicture *picture;
 
-  picture = schro_queue_find (decoder->picture_queue,
-      decoder->next_frame_number);
-  if (!picture && !decoder->end_of_stream &&
+  if (decoder->coded_order) {
+    picture = schro_queue_peek (decoder->picture_queue);
+  } else {
+    picture = schro_queue_find (decoder->picture_queue,
+        decoder->next_frame_number);
+  }
+  if (!picture && !decoder->flushing &&
       schro_queue_is_full (decoder->picture_queue)) {
+    SCHRO_ERROR("failed to find picture %d", decoder->next_frame_number);
     schro_decoder_error(decoder, "next picture not available in full queue");
     return FALSE;
   }
@@ -464,10 +488,14 @@ schro_decoder_pull (SchroDecoder *decoder)
   SCHRO_DEBUG("searching for frame %d", decoder->next_frame_number);
 
   schro_async_lock (decoder->async);
-  picture = schro_queue_find (decoder->picture_queue, decoder->next_frame_number);
+  if (decoder->coded_order) {
+    picture = schro_queue_peek (decoder->picture_queue);
+  } else {
+    picture = schro_queue_find (decoder->picture_queue, decoder->next_frame_number);
+  }
   if (picture) {
     if (picture->state & SCHRO_DECODER_STATE_DONE) {
-      schro_queue_remove (decoder->picture_queue, decoder->next_frame_number);
+      schro_queue_remove (decoder->picture_queue, picture->picture_number);
     } else {
       picture = NULL;
     }
@@ -509,25 +537,46 @@ schro_decoder_push_ready (SchroDecoder *decoder)
   return (ret == FALSE);
 }
 
+int
+schro_decoder_need_output_frame (SchroDecoder *decoder)
+{
+  if (decoder->have_sequence_header &&
+      schro_queue_is_empty (decoder->output_queue)) {
+    return TRUE;
+  }
+  return FALSE;
+}
+
 static int
 schro_decoder_get_status_locked (SchroDecoder *decoder)
 {
+  SchroPicture *next_picture;
+
   if (schro_decoder_pull_is_ready_locked (decoder)) {
     return SCHRO_DECODER_OK;
   }
   if (decoder->error) {
     return SCHRO_DECODER_ERROR;
   }
-  if (decoder->have_access_unit &&
+  if (decoder->have_sequence_header &&
       schro_queue_is_empty (decoder->output_queue)) {
     return SCHRO_DECODER_NEED_FRAME;
   }
-  if (!schro_queue_is_full (decoder->picture_queue) && !decoder->end_of_stream) {
+  if (!schro_queue_is_full (decoder->picture_queue) && !decoder->flushing) {
     return SCHRO_DECODER_NEED_BITS;
   }
-  if (decoder->end_of_stream &&
-      schro_queue_find (decoder->picture_queue, decoder->next_frame_number) == NULL) {
-    return SCHRO_DECODER_EOS;
+
+  if (decoder->coded_order) {
+    next_picture = schro_queue_peek (decoder->picture_queue);
+  } else {
+    next_picture = schro_queue_find (decoder->picture_queue, decoder->next_frame_number);
+  }
+  if (decoder->flushing && next_picture == NULL) {
+    if (decoder->end_of_stream) {
+      return SCHRO_DECODER_EOS;
+    } else {
+      return SCHRO_DECODER_STALLED;
+    }
   }
 
   return SCHRO_DECODER_WAIT;
@@ -605,15 +654,25 @@ schro_decoder_wait (SchroDecoder *decoder)
 int
 schro_decoder_push_end_of_stream (SchroDecoder *decoder)
 {
+  decoder->flushing = TRUE;
   decoder->end_of_stream = TRUE;
   return SCHRO_DECODER_EOS;
 }
 
+int
+schro_decoder_set_flushing (SchroDecoder *decoder, int value)
+{
+  decoder->flushing = value;
+
+  return SCHRO_DECODER_OK;
+}
+
 int
 schro_decoder_push (SchroDecoder *decoder, SchroBuffer *buffer)
 {
   SCHRO_ASSERT(decoder->input_buffer == NULL);
 
+  decoder->flushing = FALSE;
   decoder->input_buffer = buffer;
 
   schro_unpack_init_with_data (&decoder->unpack,
@@ -625,15 +684,15 @@ schro_decoder_push (SchroDecoder *decoder, SchroBuffer *buffer)
     int ret;
 
     SCHRO_INFO ("decoding access unit");
-    if (!decoder->have_access_unit) {
-      schro_decoder_parse_access_unit(decoder);
-      decoder->have_access_unit = TRUE;
-      decoder->access_unit_buffer = schro_buffer_dup (decoder->input_buffer);
+    if (!decoder->have_sequence_header) {
+      schro_decoder_parse_sequence_header(decoder);
+      decoder->have_sequence_header = TRUE;
+      decoder->sequence_header_buffer = schro_buffer_dup (decoder->input_buffer);
 
       ret = SCHRO_DECODER_FIRST_ACCESS_UNIT;
     } else {
-      if (schro_decoder_compare_access_unit_buffer (decoder->input_buffer,
-            decoder->access_unit_buffer)) {
+      if (schro_decoder_compare_sequence_header_buffer (decoder->input_buffer,
+            decoder->sequence_header_buffer)) {
         ret = SCHRO_DECODER_OK;
       } else {
         schro_decoder_error (decoder, "access unit changed");
@@ -676,12 +735,13 @@ schro_decoder_push (SchroDecoder *decoder, SchroBuffer *buffer)
     schro_buffer_unref (decoder->input_buffer);
     decoder->input_buffer = NULL;
     decoder->end_of_stream = TRUE;
+    decoder->flushing = TRUE;
     return SCHRO_DECODER_EOS;
   }
 
   if (SCHRO_PARSE_CODE_IS_PICTURE(decoder->parse_code)) {
 
-    if (!decoder->have_access_unit) {
+    if (!decoder->have_sequence_header) {
       SCHRO_INFO ("no access unit -- dropping picture");
       schro_buffer_unref (decoder->input_buffer);
       decoder->input_buffer = NULL;
@@ -733,9 +793,11 @@ schro_decoder_iterate_picture (SchroDecoder *decoder)
   }
 
   if (picture->is_ref) {
+    schro_async_lock (decoder->async);
     schro_decoder_reference_retire (decoder,
         decoder->picture->retired_picture_number);
     schro_decoder_reference_add (decoder, picture);
+    schro_async_unlock (decoder->async);
   }
   schro_decoder_parse_picture (picture);
 
@@ -743,7 +805,8 @@ schro_decoder_iterate_picture (SchroDecoder *decoder)
     picture->skip = TRUE;
   }
 
-  if (!picture->is_ref && decoder->skip_value > decoder->skip_ratio) {
+  if (!decoder->video_format.interlaced_coding &&
+      !picture->is_ref && decoder->skip_value > decoder->skip_ratio) {
     decoder->skip_value = (1-SCHRO_SKIP_TIME_CONSTANT) * decoder->skip_value;
     SCHRO_INFO("skipping frame %d", picture->picture_number);
     SCHRO_DEBUG("skip value %g ratio %g", decoder->skip_value, decoder->skip_ratio);
@@ -796,9 +859,11 @@ schro_decoder_parse_picture (SchroPicture *picture)
   if (params->num_refs > 0) {
     SCHRO_DEBUG("inter");
 
+    schro_async_lock (picture->decoder->async);
     picture->ref0 = schro_decoder_reference_get (picture->decoder, picture->reference1);
     if (picture->ref0 == NULL) {
       picture->error = TRUE;
+      schro_async_unlock (picture->decoder->async);
       return;
     }
     schro_picture_ref (picture->ref0);
@@ -808,10 +873,12 @@ schro_decoder_parse_picture (SchroPicture *picture)
       picture->ref1 = schro_decoder_reference_get (picture->decoder, picture->reference2);
       if (picture->ref1 == NULL) {
         picture->error = TRUE;
+        schro_async_unlock (picture->decoder->async);
         return;
       }
       schro_picture_ref (picture->ref1);
     }
+    schro_async_unlock (picture->decoder->async);
 
     schro_unpack_byte_sync (unpack);
     schro_decoder_parse_picture_prediction_parameters (picture);
@@ -915,7 +982,12 @@ schro_decoder_async_schedule (SchroDecoder *decoder,
           continue;
         }
 
-        if (picture->params.mv_precision > 0 &&
+        if (
+#ifdef HAVE_CUDA
+            1 &&
+#else
+            picture->params.mv_precision > 0 &&
+#endif
             !(refpic->state & SCHRO_DECODER_STATE_UPSAMPLE)) {
           if (!render_ok) {
             refs_ready = FALSE;
@@ -1012,6 +1084,20 @@ schro_decoder_x_render_motion (SchroPicture *picture)
     SCHRO_WARNING("motion render with %p and %p", picture->ref0, picture->ref1);
     if (picture->decoder->use_cuda) {
 #ifdef HAVE_CUDA
+      int frame_width;
+      int frame_height;
+      int frame_format;
+      SchroVideoFormat *video_format = params->video_format;
+
+      frame_format = schro_params_get_frame_format (16,
+          video_format->chroma_format);
+      frame_width = ROUND_UP_POW2(video_format->width,
+          SCHRO_LIMIT_TRANSFORM_DEPTH + video_format->chroma_h_shift);
+      frame_height = ROUND_UP_POW2(video_format->height,
+          SCHRO_LIMIT_TRANSFORM_DEPTH + video_format->chroma_v_shift);
+
+      picture->mc_tmp_frame = schro_frame_new_and_alloc (picture->decoder->cuda_domain,
+          frame_format, frame_width, frame_height);
       schro_motion_render_cuda (picture->motion, picture->mc_tmp_frame);
 #else
       SCHRO_ASSERT(0);
@@ -1136,14 +1222,16 @@ schro_decoder_x_combine (SchroPicture *picture)
     if (picture->decoder->use_cuda) {
 #ifdef HAVE_CUDA
       ref = schro_frame_new_and_alloc (decoder->cuda_domain, frame_format,
-          decoder->video_format.width, decoder->video_format.height);
+          decoder->video_format.width,
+          schro_video_format_get_picture_height(&decoder->video_format));
       schro_gpuframe_convert (ref, combined_frame);
 #else
       SCHRO_ASSERT(0);
 #endif
     } else {
       ref = schro_frame_new_and_alloc (decoder->cpu_domain, frame_format,
-          decoder->video_format.width, decoder->video_format.height);
+          decoder->video_format.width,
+          schro_video_format_get_picture_height(&decoder->video_format));
       schro_frame_convert (ref, combined_frame);
     }
     picture->upsampled_frame = schro_upsampled_frame_new (ref);
@@ -1152,7 +1240,13 @@ schro_decoder_x_combine (SchroPicture *picture)
   if (picture->has_md5) {
     uint32_t state[4];
 
-    schro_frame_md5 (picture->output_picture, state);
+    /* FIXME planar_output_frame should be fixed to always be the
+     * planar representation */
+    if (SCHRO_FRAME_IS_PACKED(picture->output_picture->format)) {
+      schro_frame_md5 (picture->planar_output_frame, state);
+    } else {
+      schro_frame_md5 (picture->output_picture, state);
+    }
     if (memcmp (state, picture->md5_checksum, 16) != 0) {
       char a[33];
       char b[33];
@@ -1172,7 +1266,11 @@ schro_decoder_x_combine (SchroPicture *picture)
 void
 schro_decoder_x_upsample (SchroPicture *picture)
 {
+#ifdef HAVE_CUDA
+  schro_upsampled_gpuframe_upsample (picture->upsampled_frame);
+#else
   schro_upsampled_frame_upsample (picture->upsampled_frame);
+#endif
 }
 
 void
@@ -1207,12 +1305,13 @@ schro_decoder_check_version (int major, int minor)
   if (major == 1 && minor == 0) return TRUE;
   if (major == 2 && minor == 0) return TRUE;
   if (major == 2 && minor == 1) return TRUE;
+  if (major == 2 && minor == 2) return TRUE;
 
   return FALSE;
 }
 
 int
-schro_decoder_compare_access_unit_buffer (SchroBuffer *a, SchroBuffer *b)
+schro_decoder_compare_sequence_header_buffer (SchroBuffer *a, SchroBuffer *b)
 {
   if (a->length != b->length) return FALSE;
   if (a->length < 13) return FALSE;
@@ -1222,7 +1321,7 @@ schro_decoder_compare_access_unit_buffer (SchroBuffer *a, SchroBuffer *b)
 }
 
 void
-schro_decoder_parse_access_unit (SchroDecoder *decoder)
+schro_decoder_parse_sequence_header (SchroDecoder *decoder)
 {
   int bit;
   int index;
@@ -1242,13 +1341,9 @@ schro_decoder_parse_access_unit (SchroDecoder *decoder)
   SCHRO_DEBUG("level = %d", decoder->level);
 
   if (!schro_decoder_check_version (decoder->major_version, decoder->minor_version)) {
-    SCHRO_ERROR("Stream version number %d:%d not handled, expecting 0:20071203, 1:0, 2:0, or 2:1",
+    SCHRO_WARNING("Stream version number %d:%d not handled, expecting 0:20071203, 1:0, 2:0, 2:1, or 2:2",
         decoder->major_version, decoder->minor_version);
   }
-  if (decoder->profile != 0 || decoder->level != 0) {
-    SCHRO_ERROR("Expecting profile/level 0:0, got %d:%d",
-        decoder->profile, decoder->level);
-  }
 
   /* base video format */
   index = schro_unpack_decode_uint (unpack);
@@ -1389,10 +1484,7 @@ schro_decoder_parse_access_unit (SchroDecoder *decoder)
     }
   }
 
-  decoder->interlaced_coding = schro_unpack_decode_uint (unpack);
-  if (decoder->interlaced_coding != 0) {
-    SCHRO_ERROR("Decoder doesn't handle interlaced coding");
-  }
+  format->interlaced_coding = schro_unpack_decode_uint (unpack);
 
   MARKER();
 
@@ -2461,18 +2553,25 @@ schro_decoder_decode_subband (SchroPicture *picture,
   if (!params->is_noarith) {
     schro_arith_decode_flush (ctx->arith);
     if (ctx->arith->offset < ctx->subband_length) {
-      SCHRO_ERROR("arith decoding didn't consume buffer (%d < %d)",
+      SCHRO_WARNING("arith decoding didn't consume buffer (%d < %d)",
           ctx->arith->offset, ctx->subband_length);
+      ctx->broken = TRUE;
     }
     if (ctx->arith->offset > ctx->subband_length + 4) {
-      SCHRO_ERROR("arith decoding overran buffer (%d > %d)",
+      SCHRO_WARNING("arith decoding overran buffer (%d > %d)",
           ctx->arith->offset, ctx->subband_length);
+      ctx->broken = TRUE;
     }
     schro_arith_free (ctx->arith);
   } else {
     /* FIXME check noarith decoding */
   }
 
+  if (ctx->broken && ctx->position != 0) {
+    schro_decoder_zero_block (ctx, 0, 0,
+        ctx->frame_data->width, ctx->frame_data->height);
+  }
+
   if (ctx->position == 0 && picture->params.num_refs == 0) {
     schro_decoder_subband_dc_predict (ctx->frame_data);
   }
@@ -2486,6 +2585,7 @@ schro_decoder_reference_add (SchroDecoder *decoder, SchroPicture *picture)
   SCHRO_DEBUG("adding %d", picture->picture_number);
 
   if (schro_queue_is_full(decoder->reference_queue)) {
+    SCHRO_ERROR("auto-retiring reference picture");
     schro_queue_pop (decoder->reference_queue);
   }
   schro_queue_add (decoder->reference_queue, schro_picture_ref(picture),
