@@ -2,9 +2,7 @@
 #ifndef _SCHRO_DOMAIN_H_
 #define _SCHRO_DOMAIN_H_
 
-#include <pthread.h>
-
-typedef int SchroExecDomain;
+#include <schroedinger/schroasync.h>
 
 typedef struct _SchroMemoryDomain SchroMemoryDomain;
 
@@ -13,7 +11,7 @@ typedef struct _SchroMemoryDomain SchroMemoryDomain;
 #define SCHRO_MEMORY_DOMAIN_SLOTS 1000
 
 struct _SchroMemoryDomain {
-  pthread_mutex_t mutex;
+  SchroMutex * mutex;
 
   unsigned int flags;
 
