@@ -121,7 +121,7 @@ schro_frame_new_and_alloc (SchroMemoryDomain *domain, SchroFrameFormat format,
         frame->components[0].length +
         frame->components[1].length + frame->components[2].length);
   } else {
-    frame->regions[0] = schro_malloc (frame->components[0].length +
+    frame->regions[0] = malloc (frame->components[0].length +
         frame->components[1].length + frame->components[2].length);
   }
 
@@ -176,6 +176,27 @@ schro_frame_new_from_data_UYVY (void *data, int width, int height)
   return frame;
 }
 
+SchroFrame *
+schro_frame_new_from_data_UYVY_full (void *data, int width, int height, int stride)
+{
+  SchroFrame *frame = schro_frame_new();
+
+  frame->format = SCHRO_FRAME_FORMAT_UYVY;
+
+  frame->width = width;
+  frame->height = height;
+
+  frame->components[0].width = width;
+  frame->components[0].height = height;
+  frame->components[0].stride = stride;
+  frame->components[0].data = data;
+  frame->components[0].length = frame->components[0].stride * height;
+  frame->components[0].v_shift = 0;
+  frame->components[0].h_shift = 0;
+
+  return frame;
+}
+
 SchroFrame *
 schro_frame_new_from_data_AYUV (void *data, int width, int height)
 {
@@ -320,7 +341,7 @@ schro_frame_unref (SchroFrame *frame)
       if (frame->domain) {
         schro_memory_domain_memfree(frame->domain, frame->regions[0]);
       } else {
-        schro_free (frame->regions[0]);
+        free (frame->regions[0]);
       }
     }
 
@@ -846,7 +867,11 @@ schro_frame_add_s16_s16 (SchroFrame *dest, SchroFrame *src)
     height = (dcomp->height < scomp->height) ? dcomp->height : scomp->height;
 
     for(y=0;y<height;y++){
+#ifdef HAVE_ORC
+      orc_add_s16 (ddata, ddata, sdata, width);
+#else
       oil_add_s16 (ddata, ddata, sdata, width);
+#endif
       ddata = OFFSET(ddata, dcomp->stride);
       sdata = OFFSET(sdata, scomp->stride);
     }
@@ -1583,6 +1608,32 @@ schro_frame_data_get_codeblock (SchroFrameData *dest, SchroFrameData *src,
   dest->v_shift = src->v_shift;
 }
 
+void
+schro_frame_split_fields (SchroFrame *dest1, SchroFrame *dest2,
+    SchroFrame *src)
+{
+  SchroFrame src_tmp;
+
+  SCHRO_ASSERT((src->height & 1) == 0);
+
+  memcpy (&src_tmp, src, sizeof(src_tmp));
+
+  src_tmp.height = src->height/2;
+  src_tmp.components[0].stride *= 2;
+  src_tmp.components[1].stride *= 2;
+  src_tmp.components[2].stride *= 2;
+
+  schro_frame_convert (dest1, &src_tmp);
+
+  src_tmp.components[0].data = OFFSET(src_tmp.components[0].data,
+      src->components[0].stride);
+  src_tmp.components[1].data = OFFSET(src_tmp.components[1].data,
+      src->components[1].stride);
+  src_tmp.components[2].data = OFFSET(src_tmp.components[2].data,
+      src->components[2].stride);
+
+  schro_frame_convert (dest2, &src_tmp);
+}
 
 /* upsampled frame */
 
