@@ -176,8 +176,8 @@ get_pixel (SchroMotion *motion, int k, SchroUpsampledFrame *upframe,
   int px, py;
 
   if (k > 0) {
-    dx >>= motion->params->video_format->chroma_h_shift;
-    dy >>= motion->params->video_format->chroma_v_shift;
+    dx >>= SCHRO_CHROMA_FORMAT_H_SHIFT(motion->params->video_format->chroma_format);
+    dy >>= SCHRO_CHROMA_FORMAT_V_SHIFT(motion->params->video_format->chroma_format);
   }
 
   px = (x << motion->mv_precision) + dx;
@@ -279,8 +279,8 @@ get_block (SchroMotion *motion, int k, int ref, int i, int j, int dx, int dy)
   SchroUpsampledFrame *upframe;
 
   if (k > 0) {
-    dx >>= motion->params->video_format->chroma_h_shift;
-    dy >>= motion->params->video_format->chroma_v_shift;
+    dx >>= SCHRO_CHROMA_FORMAT_H_SHIFT(motion->params->video_format->chroma_format);
+    dy >>= SCHRO_CHROMA_FORMAT_V_SHIFT(motion->params->video_format->chroma_format);
   }
   if (ref) {
     upframe = motion->src2;
@@ -680,15 +680,19 @@ schro_motion_render (SchroMotion *motion, SchroFrame *dest)
       motion->ybsep = params->ybsep_luma;
       motion->xblen = params->xblen_luma;
       motion->yblen = params->yblen_luma;
-      motion->width = motion->params->video_format->width;
-      motion->height = motion->params->video_format->height;
+      motion->width = comp->width;
+      motion->height = comp->height;
     } else {
-      motion->xbsep = params->xbsep_luma >> params->video_format->chroma_h_shift;
-      motion->ybsep = params->ybsep_luma >> params->video_format->chroma_v_shift;
-      motion->xblen = params->xblen_luma >> params->video_format->chroma_h_shift;
-      motion->yblen = params->yblen_luma >> params->video_format->chroma_v_shift;
-      motion->width = motion->params->video_format->width >> params->video_format->chroma_h_shift;
-      motion->height = motion->params->video_format->height >> params->video_format->chroma_v_shift;
+      motion->xbsep = params->xbsep_luma >>
+        SCHRO_CHROMA_FORMAT_H_SHIFT(motion->params->video_format->chroma_format);
+      motion->ybsep = params->ybsep_luma >>
+        SCHRO_CHROMA_FORMAT_V_SHIFT(motion->params->video_format->chroma_format);
+      motion->xblen = params->xblen_luma >>
+        SCHRO_CHROMA_FORMAT_H_SHIFT(motion->params->video_format->chroma_format);
+      motion->yblen = params->yblen_luma >>
+        SCHRO_CHROMA_FORMAT_V_SHIFT(motion->params->video_format->chroma_format);
+      motion->width = comp->width;
+      motion->height = comp->height;
     }
     motion->xoffset = (motion->xblen - motion->xbsep)/2;
     motion->yoffset = (motion->yblen - motion->ybsep)/2;
@@ -777,10 +781,10 @@ schro_motion_render (SchroMotion *motion, SchroFrame *dest)
           SCHRO_FRAME_DATA_GET_LINE(comp, j), as, motion->width);
     }
 
-    free (motion->block.data);
-    free (motion->obmc_weight.data);
-    free (motion->tmp_block_ref[0].data);
-    free (motion->tmp_block_ref[1].data);
+    schro_free (motion->block.data);
+    schro_free (motion->obmc_weight.data);
+    schro_free (motion->tmp_block_ref[0].data);
+    schro_free (motion->tmp_block_ref[1].data);
   }
 
 }
@@ -1026,6 +1030,10 @@ schro_motion_verify (SchroMotion *motion)
       mv = &motion->motion_vectors[y*params->x_num_blocks + x];
       sbmv = &motion->motion_vectors[(y&~3)*params->x_num_blocks + (x&~3)];
 
+      SCHRO_DEBUG("verify %d %d: %d %d  %d %d %d %d  %d %d %d %d", x, y, mv->split, mv->pred_mode,
+          mv->using_global, mv->unused, mv->scan, mv->metric,
+          mv->dx[0], mv->dy[0], mv->dx[1], mv->dy[1]);
+
       if (mv->split != sbmv->split) {
         SCHRO_ERROR("mv(%d,%d) has the wrong split", x, y);
         return 0;
@@ -1118,10 +1126,14 @@ schro_motion_render_ref (SchroMotion *motion, SchroFrame *dest)
       motion->xblen = params->xblen_luma;
       motion->yblen = params->yblen_luma;
     } else {
-      motion->xbsep = params->xbsep_luma >> params->video_format->chroma_h_shift;
-      motion->ybsep = params->ybsep_luma >> params->video_format->chroma_v_shift;
-      motion->xblen = params->xblen_luma >> params->video_format->chroma_h_shift;
-      motion->yblen = params->yblen_luma >> params->video_format->chroma_v_shift;
+      motion->xbsep = params->xbsep_luma >>
+        SCHRO_CHROMA_FORMAT_H_SHIFT(motion->params->video_format->chroma_format);
+      motion->ybsep = params->ybsep_luma >>
+        SCHRO_CHROMA_FORMAT_V_SHIFT(motion->params->video_format->chroma_format);
+      motion->xblen = params->xblen_luma >>
+        SCHRO_CHROMA_FORMAT_H_SHIFT(motion->params->video_format->chroma_format);
+      motion->yblen = params->yblen_luma >>
+        SCHRO_CHROMA_FORMAT_V_SHIFT(motion->params->video_format->chroma_format);
     }
     motion->xoffset = (motion->xblen - motion->xbsep)/2;
     motion->yoffset = (motion->yblen - motion->ybsep)/2;
