@@ -107,9 +107,7 @@ schro_encoder_motion_predict (SchroEncoderFrame *frame)
 
   me = schro_motionest_new (frame);
 
-  frame->motion = schro_motion_new (params,
-      frame->ref_frame[0]->reconstructed_frame,
-      (params->num_refs > 1) ? frame->ref_frame[1]->reconstructed_frame : NULL);
+  frame->motion = schro_motion_new (params, NULL, NULL);
   me->motion = frame->motion;
 
   frame->motion_field_list = schro_list_new_full ((SchroListFreeFunc)schro_motion_field_free, NULL);
@@ -136,6 +134,8 @@ schro_encoder_motion_predict (SchroEncoderFrame *frame)
 
   schro_list_free (frame->motion_field_list);
 
+  frame->badblock_ratio = (double)me->badblocks/(params->x_num_blocks*params->y_num_blocks/16);
+
   schro_motionest_free (me);
 }
 
@@ -686,8 +686,8 @@ schro_encoder_dc_estimation (SchroMotionEst *me)
 
   luma_w = params->xbsep_luma;
   luma_h = params->xbsep_luma;
-  chroma_w = luma_w>>params->video_format->chroma_h_shift;
-  chroma_h = luma_h>>params->video_format->chroma_v_shift;
+  chroma_w = luma_w>>SCHRO_CHROMA_FORMAT_H_SHIFT(params->video_format->chroma_format);
+  chroma_h = luma_h>>SCHRO_CHROMA_FORMAT_V_SHIFT(params->video_format->chroma_format);
 
   for(j=0;j<params->y_num_blocks;j++){
     for(i=0;i<params->x_num_blocks;i++){
@@ -781,6 +781,8 @@ schro_motionest_rough_scan_nohint (SchroMotionEst *me, int shift, int ref,
 
       scan.x = (i>>shift) * params->xbsep_luma;
       scan.y = (j>>shift) * params->ybsep_luma;
+      scan.block_width = MIN(scan.frame->width - scan.x, params->xbsep_luma);
+      scan.block_height = MIN(scan.frame->height - scan.y, params->ybsep_luma);
       schro_metric_scan_setup (&scan, 0, 0, distance);
 
       mv = motion_field_get (mf, i, j);
@@ -790,6 +792,17 @@ schro_motionest_rough_scan_nohint (SchroMotionEst *me, int shift, int ref,
         mv->metric = SCHRO_METRIC_INVALID;
         continue;
       }
+#if 0
+      /* this code skips blocks that are off the edge.  Instead, we
+       * scan smaller block sizes */
+      if (scan.x + scan.block_width >= scan.ref_frame->width ||
+          scan.y + scan.block_height >= scan.ref_frame->height) {
+        mv->dx[ref] = 0 << shift;
+        mv->dy[ref] = 0 << shift;
+        mv->metric = SCHRO_METRIC_INVALID;
+        continue;
+      }
+#endif
 
       schro_metric_scan_do_scan (&scan);
       mv->metric = schro_metric_scan_get_min (&scan, &dx, &dy);
@@ -798,6 +811,8 @@ schro_motionest_rough_scan_nohint (SchroMotionEst *me, int shift, int ref,
 
       mv->dx[ref] = dx;
       mv->dy[ref] = dy;
+
+      me->hier_score += (mv->metric>10*12*12);
     }
   }
 
@@ -918,6 +933,8 @@ schro_motionest_rough_scan_hint (SchroMotionEst *me, int shift, int ref,
 
       scan.x = (i>>shift) * params->xbsep_luma;
       scan.y = (j>>shift) * params->ybsep_luma;
+      scan.block_width = MIN(scan.frame->width - scan.x, params->xbsep_luma);
+      scan.block_height = MIN(scan.frame->height - scan.y, params->ybsep_luma);
       schro_metric_scan_setup (&scan, dx, dy, distance);
 
       mv = motion_field_get (mf, i, j);
@@ -1058,8 +1075,8 @@ schro_motionest_superblock_dc (SchroMotionEst *me,
   metric = schro_block_average (&mvdc->dc[0], frame->components + 0,
       i * params->xbsep_luma, j * params->ybsep_luma,
       4 * params->xbsep_luma, 4 * params->ybsep_luma);
-  chroma_w = params->xbsep_luma>>params->video_format->chroma_h_shift;
-  chroma_h = params->xbsep_luma>>params->video_format->chroma_v_shift;
+  chroma_w = params->xbsep_luma>>SCHRO_CHROMA_FORMAT_H_SHIFT(params->video_format->chroma_format);
+  chroma_h = params->ybsep_luma>>SCHRO_CHROMA_FORMAT_V_SHIFT(params->video_format->chroma_format);
   schro_block_average (&mvdc->dc[1], frame->components + 1,
       i * chroma_w, j * chroma_h, 4 * chroma_w, 4 * chroma_h);
   schro_block_average (&mvdc->dc[2], frame->components + 2,
@@ -1217,68 +1234,83 @@ schro_encoder_bigblock_estimation (SchroMotionEst *me)
 {
   SchroParams *params = me->params;
   int i,j;
-  int total_error = 0;
+  double total_error = 0;
+  int block_size;
 
-  me->lambda = 0.1;
+  me->lambda = me->encoder_frame->encoder->magic_mc_lambda;
+
+  block_size = 16 * params->xbsep_luma * params->ybsep_luma;
 
   for(j=0;j<params->y_num_blocks;j+=4){
     for(i=0;i<params->x_num_blocks;i+=4){
-      SchroBlock blocks[10];
-      int n;
+      SchroBlock block = { 0 };
+      SchroBlock tryblock = { 0 };
       double score;
-      int k;
       double min_score;
-      int min_k;
+      double min_score1;
 
-      memset (blocks, 0, sizeof(blocks));
+#define TRYBLOCK \
+      score = tryblock.entropy + me->lambda * tryblock.error; \
+      if (tryblock.valid && score < min_score) { \
+        memcpy (&block, &tryblock, sizeof(block)); \
+        min_score = score; \
+      }
 
-      n = 0;
-      schro_motionest_superblock_scan_one (me, 0, 4, blocks + n, i, j);
-      n++;
-      schro_motionest_superblock_predicted (me, 0, blocks + n, i, j);
-      n++;
-      schro_motionest_block_scan_one (me, 0, 12, blocks + n, i, j);
-      n++;
-      schro_motionest_superblock_dc (me, blocks + n, i, j);
-      n++;
-      schro_motionest_superblock_dc_predicted (me, blocks + n, i, j);
-      n++;
+      /* base 119 s */
+      schro_motionest_superblock_predicted (me, 0, &block, i, j);
+      min_score = block.entropy + me->lambda * block.error;
       if (params->num_refs > 1) {
-        schro_motionest_superblock_scan_one (me, 1, 4, blocks + n, i, j);
-        n++;
-        schro_motionest_superblock_predicted (me, 1, blocks + n, i, j);
-        n++;
-        schro_motionest_superblock_biref_zero (me, blocks + n, i, j);
-        n++;
-        schro_motionest_block_scan_one (me, 1, 12, blocks + n, i, j);
-        n++;
+        schro_motionest_superblock_predicted (me, 1, &tryblock, i, j);
+        TRYBLOCK
       }
 
-      min_k = 0;
-      min_score = blocks[0].entropy + me->lambda * blocks[0].error;
-      for(k=0;k<n;k++){
-#if 0
-        if (blocks[k].error == SCHRO_METRIC_INVALID) {
-          SCHRO_ERROR("k %d", k);
-          SCHRO_ASSERT(blocks[k].error != SCHRO_METRIC_INVALID);
-        }
-        SCHRO_ASSERT(blocks[k].error != 16*SCHRO_METRIC_INVALID);
-#endif
-        if (!blocks[k].valid) continue;
-        score = blocks[k].entropy + me->lambda * blocks[k].error;
-        if (score < min_score) {
-          min_k = k;
-          min_score = score;
+      /* 16 s */
+      schro_motionest_superblock_scan_one (me, 0, 4, &tryblock, i, j);
+      TRYBLOCK
+      if (params->num_refs > 1) {
+        schro_motionest_superblock_scan_one (me, 1, 4, &tryblock, i, j);
+        TRYBLOCK
+      }
+
+      /* 2.5 s */
+      schro_motionest_superblock_dc_predicted (me, &tryblock, i, j);
+      TRYBLOCK
+      schro_motionest_superblock_dc (me, &tryblock, i, j);
+      TRYBLOCK
+
+      /* 3.0 s */
+      if (params->num_refs > 1) {
+        schro_motionest_superblock_biref_zero (me, &tryblock, i, j);
+        TRYBLOCK
+      }
+
+      if (min_score > 1000) {
+        min_score1 = min_score;
+        schro_motionest_block_scan_one (me, 0, 4, &tryblock, i, j);
+        TRYBLOCK
+        //schro_dump(SCHRO_DUMP_MOTIONEST, "%g %g %g\n", min_score1, min_score, score);
+        if (params->num_refs > 1) {
+          schro_motionest_block_scan_one (me, 1, 4, &tryblock, i, j);
+          TRYBLOCK
         }
       }
-      schro_block_fixup (blocks + min_k);
-      schro_motion_copy_to (me->motion, i, j, blocks + min_k);
+
+      if (block.error > 10*block_size) {
+        me->badblocks++;
+      }
+
+      schro_block_fixup (&block);
+      schro_motion_copy_to (me->motion, i, j, &block);
       
-      total_error += blocks[min_k].error;
+      total_error += (double)block.error*block.error/(double)(block_size * block_size);
     }
   }
 
-  me->encoder_frame->mc_error = total_error;
+  me->encoder_frame->mc_error = total_error/(240.0*240.0)/
+    (params->x_num_blocks*params->y_num_blocks/16);
+
+  /* magic parameter */
+  me->encoder_frame->mc_error *= 2.5;
 }
 
 int
