@@ -14,8 +14,13 @@ void schro_encoder_choose_quantisers_simple (SchroEncoderFrame *frame);
 void schro_encoder_choose_quantisers_rate_distortion (SchroEncoderFrame *frame);
 void schro_encoder_choose_quantisers_lossless (SchroEncoderFrame *frame);
 void schro_encoder_choose_quantisers_lowdelay (SchroEncoderFrame *frame);
+void schro_encoder_choose_quantisers_constant_lambda (SchroEncoderFrame *frame);
+void schro_encoder_choose_quantisers_constant_error (SchroEncoderFrame *frame);
 
 double schro_encoder_entropy_to_lambda (SchroEncoderFrame *frame, double entropy);
+static double schro_encoder_lambda_to_entropy (SchroEncoderFrame *frame, double lambda);
+static void schro_encoder_generate_subband_histograms (SchroEncoderFrame *frame);
+
 
 static int schro_subband_pick_quant (SchroEncoderFrame *frame,
     int component, int i, double lambda);
@@ -23,24 +28,39 @@ static int schro_subband_pick_quant (SchroEncoderFrame *frame,
 #define CURVE_SIZE 128
 
 double
-schro_encoder_perceptual_weight_moo (double ppd)
+schro_encoder_perceptual_weight_moo (double cpd)
 {
   /* I pretty much pulled this out of my ass (ppd = pixels per degree) */
-  if (ppd < 8) return 1;
-  return 1.0/(0.34 * ppd * exp (-0.125 * ppd));
+  if (cpd < 4) return 1;
+  return 0.68 * cpd * exp (-0.25 * cpd);
 }
 
 double
-schro_encoder_perceptual_weight_constant (double ppd)
+schro_encoder_perceptual_weight_constant (double cpd)
 {
   return 1;
 }
 
 double
-schro_encoder_perceptual_weight_ccir959 (double ppd)
+schro_encoder_perceptual_weight_ccir959 (double cpd)
 {
-  ppd *= 0.5;
-  return 0.255 / pow(1 + 0.2561 * ppd * ppd, -0.75);
+  double w;
+  w = 0.255 * pow(1 + 0.2561 * cpd * cpd, -0.75);
+
+  /* return normalized value */
+  return w/0.255;
+}
+
+double
+schro_encoder_perceptual_weight_manos_sakrison (double cpd)
+{
+  double w;
+
+  if (cpd < 4) return 1;
+  w = 2.6 * (0.0192 + 0.114*cpd) * exp(-pow(0.114*cpd,1.1));
+
+  /* return normalized value */
+  return w / 0.980779694777866;
 }
 
 static double
@@ -136,8 +156,8 @@ schro_encoder_calculate_subband_weights (SchroEncoder *encoder,
 
   for(j=0;j<CURVE_SIZE;j++){
     for(i=0;i<CURVE_SIZE;i++){
-      double fv = j*encoder->pixels_per_degree_vert*(1.0/CURVE_SIZE);
-      double fh = i*encoder->pixels_per_degree_horiz*(1.0/CURVE_SIZE);
+      double fv = j*encoder->cycles_per_degree_vert*(1.0/CURVE_SIZE);
+      double fh = i*encoder->cycles_per_degree_horiz*(1.0/CURVE_SIZE);
 
       weight[j*CURVE_SIZE+i] = perceptual_weight (sqrt(fv*fv+fh*fh));
     }
@@ -266,8 +286,11 @@ schro_encoder_choose_quantisers (SchroEncoderFrame *frame)
     case SCHRO_QUANTISER_ENGINE_LOWDELAY:
       schro_encoder_choose_quantisers_lowdelay (frame);
       break;
-    case SCHRO_QUANTISER_ENGINE_RATE_DISTORTION_2:
-      //schro_encoder_choose_quantisers_rate_distortion_2 (frame);
+    case SCHRO_QUANTISER_ENGINE_CONSTANT_LAMBDA:
+      schro_encoder_choose_quantisers_constant_lambda (frame);
+      break;
+    case SCHRO_QUANTISER_ENGINE_CONSTANT_ERROR:
+      schro_encoder_choose_quantisers_constant_error (frame);
       break;
   }
 }
@@ -289,7 +312,6 @@ void
 schro_encoder_choose_quantisers_simple (SchroEncoderFrame *frame)
 {
   SchroParams *params = &frame->params;
-  int psnr;
   int i;
   int component;
   double noise_amplitude;
@@ -297,9 +319,7 @@ schro_encoder_choose_quantisers_simple (SchroEncoderFrame *frame)
   double max;
   double *table;
 
-  psnr = frame->encoder->noise_threshold;
-
-  noise_amplitude = 255.0 * pow(0.1, psnr*0.05);
+  noise_amplitude = 255.0 * pow(0.1, frame->encoder->noise_threshold*0.05);
   SCHRO_DEBUG("noise %g", noise_amplitude);
 
   table = frame->encoder->subband_weights[params->wavelet_filter_index]
@@ -334,15 +354,13 @@ void
 schro_encoder_choose_quantisers_lowdelay (SchroEncoderFrame *frame)
 {
   SchroParams *params = &frame->params;
-  int psnr;
   int i;
   int component;
   int base;
   const int *table;
 
-  psnr = frame->encoder->noise_threshold;
   /* completely made up */
-  base = 12 + (30 - psnr)/2;
+  base = 12 + (30 - frame->encoder->noise_threshold)/2;
 
   table = schro_tables_lowdelay_quants[params->wavelet_filter_index]
       [params->transform_depth-1];
@@ -359,7 +377,6 @@ schro_encoder_choose_quantisers_lowdelay (SchroEncoderFrame *frame)
 
 }
 
-
 #ifdef DUMP_SUBBAND_CURVES
 static double pow2(double x)
 {
@@ -604,6 +621,7 @@ schro_encoder_generate_subband_histograms (SchroEncoderFrame *frame)
           &frame->subband_hists[component][i], skip);
     }
   }
+
   frame->have_histograms = TRUE;
 }
 
@@ -699,23 +717,25 @@ schro_encoder_calc_estimates (SchroEncoderFrame *frame)
 void
 schro_encoder_choose_quantisers_rate_distortion (SchroEncoderFrame *frame)
 {
-  SchroParams *params = &frame->params;
-  int i;
-  int component;
+  //SchroParams *params = &frame->params;
+  //int i;
+  //int component;
   double base_lambda;
   int bits;
+  double ratio;
 
   schro_encoder_generate_subband_histograms (frame);
   schro_encoder_calc_estimates (frame);
 
   SCHRO_ASSERT(frame->have_estimate_tables);
 
-  {
-    double scale = 1/frame->encoder->average_arith_context_ratio;
-    if (scale < 0.8) scale = 0.8;
-    if (scale > 2.0) scale = 2.0;
-    bits = frame->allocated_residual_bits * scale;
+  if (frame->num_refs == 0) {
+    ratio = frame->encoder->average_arith_context_ratio_intra;
+  } else {
+    ratio = frame->encoder->average_arith_context_ratio_inter;
   }
+  frame->estimated_arith_context_ratio = CLAMP(ratio, 0.5, 1.2);
+  bits = frame->allocated_residual_bits;
 
   base_lambda = schro_encoder_entropy_to_lambda (frame, bits);
 #if 0
@@ -740,23 +760,15 @@ schro_encoder_choose_quantisers_rate_distortion (SchroEncoderFrame *frame)
   }
 #endif
   frame->base_lambda = base_lambda;
-  SCHRO_DEBUG("LAMBDA: %d %g", frame->frame_number, base_lambda);
+  SCHRO_DEBUG("LAMBDA: %d %g %d", frame->frame_number, base_lambda, bits);
 
+  schro_encoder_lambda_to_entropy (frame, base_lambda);
+#if 0
   for(component=0;component<3;component++){
     for(i=0;i<1 + 3*params->transform_depth; i++) {
-      int vol;
-      int16_t *data;
-      int stride;
-      int width, height;
       double lambda;
-      int position;
       double weight;
 
-      position = schro_subband_get_position (i);
-      schro_subband_get (frame->iwt_frame, component, position,
-          &frame->params, &data, &stride, &width, &height);
-      vol = width * height;
-
       lambda = base_lambda;
       if (i == 0) {
         lambda *= frame->encoder->magic_subband0_lambda_scale;
@@ -773,6 +785,20 @@ schro_encoder_choose_quantisers_rate_distortion (SchroEncoderFrame *frame)
           component, i, lambda);
     }
   }
+#endif
+}
+
+void
+schro_encoder_choose_quantisers_constant_lambda (SchroEncoderFrame *frame)
+{
+  schro_encoder_generate_subband_histograms (frame);
+  schro_encoder_calc_estimates (frame);
+
+  SCHRO_ASSERT(frame->have_estimate_tables);
+
+  frame->base_lambda = frame->encoder->magic_lambda;
+
+  schro_encoder_lambda_to_entropy (frame, frame->base_lambda);
 }
 
 
@@ -786,12 +812,16 @@ schro_encoder_estimate_entropy (SchroEncoderFrame *frame)
 
   for(component=0;component<3;component++){
     for(i=0;i<1 + 3*params->transform_depth; i++) {
+#if 0
       n += schro_histogram_estimate_entropy (
             &frame->subband_hists[component][i],
             frame->quant_index[component][i], params->is_noarith);
+#else
+      n += frame->est_entropy[component][i][frame->quant_index[component][i]];
+#endif
     }
   }
-  frame->estimated_residual_bits = n;
+  frame->estimated_residual_bits = n * frame->estimated_arith_context_ratio;
 
   if (frame->allocated_residual_bits > 0 &&
       frame->estimated_residual_bits >
@@ -915,15 +945,24 @@ schro_encoder_lambda_to_entropy (SchroEncoderFrame *frame, double base_lambda)
 
       lambda = base_lambda;
 
+      if (i == 0) {
+        lambda *= frame->encoder->magic_subband0_lambda_scale;
+      }
+      if (component > 0) {
+        lambda *= frame->encoder->magic_chroma_lambda_scale;
+      }
+
       weight = frame->encoder->subband_weights[frame->params.wavelet_filter_index]
         [frame->params.transform_depth-1][i];
       lambda /= weight*weight;
       
       quant_index = schro_subband_pick_quant (frame, component, i, lambda);
       entropy += frame->est_entropy[component][i][quant_index];
+      frame->quant_index[component][i] = quant_index;
     }
   }
-  return entropy;
+
+  return entropy * frame->estimated_arith_context_ratio;
 }
 
 double
@@ -948,7 +987,7 @@ schro_encoder_entropy_to_lambda (SchroEncoderFrame *frame, double entropy)
 
       SCHRO_DEBUG("have: log_lambda=[%g,%g] entropy=[%g,%g] target=%g",
           log_lambda_lo, log_lambda_hi, entropy_lo, entropy_hi, entropy);
-      if (entropy_hi >= entropy) break;
+      if (entropy_hi > entropy) break;
 
       SCHRO_DEBUG("--> step up");
 
@@ -972,16 +1011,15 @@ schro_encoder_entropy_to_lambda (SchroEncoderFrame *frame, double entropy)
     }
     SCHRO_DEBUG("--> stopping");
   }
-
   if (entropy_lo == entropy_hi) {
     return exp(0.5*(log_lambda_lo + log_lambda_hi));
   }
 
-  if (!(entropy_lo <= entropy && entropy_hi >= entropy)) {
-    SCHRO_DEBUG("entropy not bracketed");
+  if (entropy_lo > entropy || entropy_hi < entropy) {
+    SCHRO_ERROR("entropy not bracketed");
   }
 
-  for(j=0;j<10;j++){
+  for(j=0;j<14;j++){
     double x;
 
     if (entropy_hi == entropy_lo) break;
@@ -989,9 +1027,13 @@ schro_encoder_entropy_to_lambda (SchroEncoderFrame *frame, double entropy)
     SCHRO_DEBUG("have: log_lambda=[%g,%g] entropy=[%g,%g] target=%g",
         log_lambda_lo, log_lambda_hi, entropy_lo, entropy_hi, entropy);
 
+#if 0
     x = (entropy - entropy_lo) / (entropy_hi - entropy_lo);
     if (x < 0.2) x = 0.2;
     if (x > 0.8) x = 0.8;
+#else
+    x = 0.5;
+#endif
     log_lambda_mid = log_lambda_lo + (log_lambda_hi - log_lambda_lo) * x;
     entropy_mid = schro_encoder_lambda_to_entropy (frame, exp(log_lambda_mid));
 
@@ -1014,3 +1056,151 @@ schro_encoder_entropy_to_lambda (SchroEncoderFrame *frame, double entropy)
   return exp(log_lambda_mid);
 }
 
+static double
+schro_encoder_lambda_to_error (SchroEncoderFrame *frame, double base_lambda)
+{
+  SchroParams *params = &frame->params;
+  int i;
+  int component;
+  double error = 0;
+
+  for(component=0;component<3;component++){
+    for(i=0;i<1 + 3*params->transform_depth; i++) {
+      double lambda;
+      double weight;
+      int quant_index;
+
+      lambda = base_lambda;
+
+      if (i == 0) {
+        lambda *= frame->encoder->magic_subband0_lambda_scale;
+      }
+      if (component > 0) {
+        lambda *= frame->encoder->magic_chroma_lambda_scale;
+      }
+
+      weight = frame->encoder->subband_weights[frame->params.wavelet_filter_index]
+        [frame->params.transform_depth-1][i];
+      lambda /= weight*weight;
+      
+      quant_index = schro_subband_pick_quant (frame, component, i, lambda);
+      error += frame->est_error[component][i][quant_index];
+      frame->quant_index[component][i] = quant_index;
+    }
+  }
+
+  return error;
+}
+
+double
+schro_encoder_error_to_lambda (SchroEncoderFrame *frame, double error)
+{
+  int j;
+  double log_lambda_hi, log_lambda_lo, log_lambda_mid;
+  double error_hi, error_lo, error_mid;
+
+  log_lambda_lo = log(1);
+  error_lo = schro_encoder_lambda_to_error (frame, exp(log_lambda_lo));
+  SCHRO_DEBUG("start target=%g log_lambda=%g error=%g",
+      error, log_lambda_lo, error_lo, log_lambda_lo, error);
+
+  if (error < error_lo) {
+    error_hi = error_lo;
+    log_lambda_hi = log_lambda_lo;
+
+    for(j=0;j<5;j++) {
+      log_lambda_lo = log_lambda_hi + log(100);
+      error_lo = schro_encoder_lambda_to_error (frame, exp(log_lambda_lo));
+
+      SCHRO_DEBUG("have: log_lambda=[%g,%g] error=[%g,%g] target=%g",
+          log_lambda_lo, log_lambda_hi, error_lo, error_hi, error);
+      if (error > error_lo) break;
+
+      SCHRO_DEBUG("--> step up");
+
+      error_hi = error_lo;
+      log_lambda_hi = log_lambda_lo;
+    }
+    SCHRO_DEBUG("--> stopping");
+  } else {
+    for(j=0;j<5;j++) {
+      log_lambda_hi = log_lambda_lo - log(100);
+      error_hi = schro_encoder_lambda_to_error (frame, exp(log_lambda_hi));
+
+      SCHRO_DEBUG("have: log_lambda=[%g,%g] error=[%g,%g] target=%g",
+          log_lambda_lo, log_lambda_hi, error_lo, error_hi, error);
+
+      SCHRO_DEBUG("--> step down");
+      if (error < error_hi) break;
+
+      error_lo = error_hi;
+      log_lambda_lo = log_lambda_hi;
+    }
+    SCHRO_DEBUG("--> stopping");
+  }
+  if (error_lo == error_hi) {
+    return exp(0.5*(log_lambda_lo + log_lambda_hi));
+  }
+
+  if (error_lo > error || error_hi < error) {
+    SCHRO_DEBUG("error not bracketed");
+  }
+
+  for(j=0;j<14;j++){
+    double x;
+
+    if (error_hi == error_lo) break;
+
+    SCHRO_DEBUG("have: log_lambda=[%g,%g] error=[%g,%g] target=%g",
+        log_lambda_lo, log_lambda_hi, error_lo, error_hi, error);
+
+#if 0
+    x = (error - error_lo) / (error_hi - error_lo);
+    if (x < 0.2) x = 0.2;
+    if (x > 0.8) x = 0.8;
+#else
+    x = 0.5;
+#endif
+    log_lambda_mid = log_lambda_lo + (log_lambda_hi - log_lambda_lo) * x;
+    error_mid = schro_encoder_lambda_to_error (frame, exp(log_lambda_mid));
+
+    SCHRO_DEBUG("picking x=%g log_lambda_mid=%g error=%g", x,
+        log_lambda_mid, error_mid);
+
+    if (error_mid > error) {
+      log_lambda_hi = log_lambda_mid;
+      error_hi = error_mid;
+      SCHRO_DEBUG("--> focus up");
+    } else {
+      log_lambda_lo = log_lambda_mid;
+      error_lo = error_mid;
+      SCHRO_DEBUG("--> focus down");
+    }
+  }
+
+  log_lambda_mid = 0.5*(log_lambda_hi + log_lambda_lo);
+  SCHRO_DEBUG("done %g", exp(log_lambda_mid));
+  return exp(log_lambda_mid);
+}
+
+void
+schro_encoder_choose_quantisers_constant_error (SchroEncoderFrame *frame)
+{
+  double base_lambda;
+  double error;
+
+  schro_encoder_generate_subband_histograms (frame);
+  schro_encoder_calc_estimates (frame);
+
+  SCHRO_ASSERT(frame->have_estimate_tables);
+
+  error = 255.0 * pow(0.1, frame->encoder->noise_threshold*0.05);
+  error *= frame->params.video_format->width *
+    frame->params.video_format->height;
+
+  base_lambda = schro_encoder_error_to_lambda (frame, error);
+
+  frame->base_lambda = base_lambda;
+  SCHRO_DEBUG("LAMBDA: %d %g", frame->frame_number, base_lambda);
+}
+
