@@ -127,6 +127,14 @@ schro_queue_pop (SchroQueue *queue)
   queue->n--;
 }
 
+void *
+schro_queue_peek (SchroQueue *queue)
+{
+  if (queue->n == 0) return NULL;
+
+  return queue->elements[0].data;
+}
+
 void *
 schro_queue_pull (SchroQueue *queue)
 {
@@ -148,6 +156,12 @@ schro_queue_is_full (SchroQueue *queue)
   return (queue->n == queue->size);
 }
 
+int
+schro_queue_slots_available (SchroQueue *queue)
+{
+  return queue->size - queue->n;
+}
+
 int
 schro_queue_is_empty (SchroQueue *queue)
 {
